import * as React from 'react'
import './index.less'
import { CanvasUtils } from './utils'
const canvasUtils = new CanvasUtils()
export interface IStateProps {
    align?: string, // 对称点 top，right，bottom，left
    bgColor?: string, // 十六进制色值 背景色
    color?: string, // 十六进制色值
    counterclockwise?: boolean, // 绘制方向，默认顺时针 true = 逆时针 ，false = 顺时针
    height: number, // 画布高
    label?: string, // 画布文字
    lineCap?: string, // 线条样式  round, butt, square
    lineSize?: number, // 线条大小
    minSpacing?: number,
    nowValue: number, // 当前角度(需换算，总角度360，(当前值/总值)*360)
    radius: number, // 半径
    allSize: number,
    type?: string, // 类型 normal，semiSymmetric
    width: number, // 画布宽
}

export default class DashboardComponent extends React.Component<IStateProps, IStateProps> {
    public static state: IStateProps
    public _VDOM: HTMLCanvasElement | null
    public _OPTION: IStateProps = {
        allSize: 360,
        height: 300,
        nowValue: 140,
        radius: 100,
        width: 300,
    }
    constructor (props: IStateProps) {
        super(props)
        this._OPTION = canvasUtils.getPropsToOption(props)
    }
    public componentDidMount() {
        // 处理绘制
        this.handleDraw()
    }
    public componentDidUpdate (prevProps: IStateProps, prevState: IStateProps) {
        // render时，重置_OPTION，重新绘制
        this._OPTION = canvasUtils.getPropsToOption(this.props)
        this.handleDraw()
    }
    public handleDraw = () => {
        if (this._VDOM) {
            const ctx = this._VDOM.getContext('2d')
            if (ctx) {
                // 获取绘制所需参数
                const opt = this.handleOpt()
                // 前景圆
                this.drawBgCircle(ctx, opt)
                // 背景圆
                this.drawCircle(ctx, opt)
            }
        }
    }
    // 前景圆
    public drawBgCircle = (ctx: CanvasRenderingContext2D, opt:any) => {
        ctx.beginPath()
        ctx.lineWidth = opt.lineSize
        ctx.strokeStyle = opt.backGroundcolor
        ctx.lineCap = opt.lineCap
        ctx.arc(opt.x, opt.y, opt.r, 0, 2 * Math.PI)
        ctx.stroke()
    }
    // 背景圆
    public drawCircle = (ctx: CanvasRenderingContext2D, opt:any) => {
        ctx.beginPath()
        ctx.lineWidth = opt.lineSize
        ctx.strokeStyle = opt.lineColor
        ctx.lineCap = opt.lineCap
        ctx.arc(opt.x, opt.y, opt.r, opt.start, opt.end)
        ctx.stroke()
    }
    // 处理绘制所需参数
    public handleOpt =() => {
        const radius: number = this._OPTION.radius
        const circleX: number = this._OPTION.width / 2
        const circleY: number = this._OPTION.height / 2
        const nowValue = this._OPTION.nowValue || 0
        const color = this._OPTION.color || '#E6A23C'
        const Bgcolor = this._OPTION.bgColor || '#eee'
        const size = this._OPTION.lineSize || 8
        const cap = this._OPTION.lineCap || 'round' 
        const type = this._OPTION.type || 'normal'
        // type 为semiSymmetric的时候，需要allSize，minSpacing这两个参数，分别是总值，最小间隔
        const allSize = this._OPTION.allSize || 270 
        const minSpacing = this._OPTION.minSpacing
        const counterclockwise = this._OPTION.counterclockwise || false
        const pos = canvasUtils.handleVDomPos(type, nowValue, this._OPTION.align, allSize, minSpacing, counterclockwise)
        return {
            backGroundcolor: Bgcolor,
            end: pos.end,
            lineCap: cap,
            lineColor: color,
            lineSize: size,
            r: radius,
            start: pos.start,
            x: circleX,
            y: circleY,
        }
    }
    public render(): JSX.Element {
        const {allSize, width, height, nowValue} = this.props
        const percent: number = nowValue / allSize
        return (
            <div className='dashboard_container'>
                <div className='sale'>
                    <p>销售额</p>
                    <p className='sale_percent'>{ (percent * 100).toFixed(2)}%</p>
                </div>
                <canvas id='2' width={width} height={height} ref={(ref) => this._VDOM = ref} />
            </div>
        )
    }
}