import { IStateProps } from '../'
export class CanvasUtils {
    /**根据方位获取顺时针对称点
     * @param {String} pos top,right,bottom,left
     * @param {String} type normal,semiSymmetric
     * @param {Boolean} counterclockwise
     */
    public getsymmetryPoint = (pos: string = 'top', type: string = 'normal', counterclockwise: boolean = false) => {
        if (type !== 'normal' && counterclockwise) {
            switch (pos) {
                case 'top':return 0.5; break
                case 'right':return 1; break
                case 'bottom':return 1.5; break
                case 'left':return 0; break
                default: throw new Error ('The Props align is Invalid!')
            }
        }
        switch (pos) {
            case 'top':return 1.5; break
            case 'right':return 0; break
            case 'bottom':return 0.5; break
            case 'left':return 1; break
            default: throw new Error ('The Props align is Invalid!')
        }
    }

    /**根据类型获取不同配置参数
     * @param {String} type normal,semiSymmetric
     * @param {Number} nowValue 当前值
     * @param {String} align top,right,bottom,left
     * @param {Number} allSize 总值
     * @param {Number} minSpacing 半圆间隔值
     */
    public handleVDomPos = (
        type: string = 'normal',
        nowValue: number,
        align: string = 'top',
        allSize:number = 270,
        minSpacing: number = 0.5,
        counterclockwise: boolean = false
    ) => {
        const symmetryPoint:number = this.getsymmetryPoint(align, type, counterclockwise)
        switch (type) {
            case 'normal':return this.getNormalPos(allSize, nowValue, symmetryPoint); break
            case 'semiSymmetric':return this.getSemiSymmetricPos(nowValue, symmetryPoint, allSize, minSpacing, counterclockwise); break
            default: return this.getNormalPos(allSize, nowValue, symmetryPoint)
        }
    }
    // 获取props作为初始参数
    public getPropsToOption = (props: IStateProps) => {
        const temp = {
            allSize: 360,
            height: 300,
            nowValue: 140,
            radius: 100,
            width: 300,
        }
        for (const i in props) {
            if (true) {
                temp[i] = props[''+ i +'']
            }
        }
        return temp
    }
    /**获取默认配置参数
     * @param {Number} allSize 总值
     * @param {Number} nowValue 当前值
     * @param {Number} symmetryPoint 对称点
     */
    private getNormalPos = (allSize: number, nowValue: number, symmetryPoint: number) => {
        // 起点
        let _START: number = 0
        // 终点
        let endPoint: number = 360
        // 需绘制弧长
        let r: number = 0
        if (nowValue && nowValue > allSize || nowValue < 0) {
            this.throwError('The props nowValue is bettwen 0 and your allSize!')
        }
        // 需绘制弧长
        r =  2 * (nowValue / allSize)
        // 设置绘图起点
        _START = (symmetryPoint - r / 2) * Math.PI
        // 设置绘图终点
        endPoint = (symmetryPoint + r / 2) * Math.PI
        return {
            end: endPoint,
            start: _START,
        }
    }
    /**获取间隔圆参数
     * @param {Number} nowValue 总值
     * @param {Number} symmetryPoint 当前值
     * @param {Number} allSize 对称点
     * @param {Number} minSpacing 间隔值
     */
    private getSemiSymmetricPos = (nowValue: number, symmetryPoint: number, allSize: number, minSpacing: number, counterclockwise: boolean) => {
        console.log(minSpacing)
        console.log(symmetryPoint)
        // 
        // 默认总角度270
        const baseDeg: number = 270
        // 当前值所占角度，默认总角度270
        const nowDeg = (nowValue / allSize) * baseDeg
        // 起点
        let _START: number = 0
        // 终点
        let endPoint: number = 0
        // 需绘制弧长
        let radian: number = 0
        if (nowValue && nowValue > allSize || nowValue < 0) {
            this.throwError('The props nowValue is bettwen 0 and your allSize!')
        }
        if (minSpacing > 2 || nowValue < 0) {
            this.throwError('The props minSpacing is bettwen 0 and 2!')
        }
        // 设置绘图起点 
        _START = (symmetryPoint+ minSpacing / 2) * Math.PI
        // 需绘制弧长
        radian =  (2 - minSpacing) * (nowDeg / baseDeg) * Math.PI
        // 设置绘图终点
        endPoint = radian + _START
        if (counterclockwise) {
            console.log('逆时针，未完成')
        }
        return {
            end: endPoint,
            start: _START,
        }
    }
    // 抛错
    private throwError (msg:string = '') {
        return new Error(msg)
    }
}
